#ifndef __CONSOLA_H__
#define __CONSOLA_H__

#include "mmu.h"

#define CONS_ALTO	25
#define CONS_ANCHO	80
#define CONS_MARGEN_TOP	2
#define CONS_MARGEN_BOT	2
#define CONS_TEXTO_ALTO	(CONS_ALTO - CONS_MARGEN_TOP - CONS_MARGEN_BOT)
#define CONS_TEXTO_AREA	(CONS_TEXTO_ALTO * CONS_ANCHO)

#define CHAR_BLANK	(' ' | 0x07 << 8)
#define MODO_DEFAULT	0x07
#define COL_BANDA1	COLOR(FG_LIGHT(COL_GRIS), COL_VERDE)
#define COL_BANDA2	COLOR(FG_LIGHT(COL_GRIS), COL_AZUL)

// Colores de 16 bits
#define FG_LIGHT(x)	((x) | 0x8)
#define BG_BLINK(x)	((x) | 0x8)
#define COL_NEGRO	0
#define COL_AZUL	1
#define COL_VERDE	2
#define COL_CIAN	3
#define COL_ROJO	4
#define COL_MAGENTA	5
#define COL_MARRON	6
#define COL_GRIS	7
#define COLOR(f,b)	(((b) << 4) | (f))

#define MAX_CONSOLAS	12
#define PAGINAS_CONSOLA	4
// Queremos que sea multiplo del tamanio de linea, asi no se corre al hacer scroll
#define CONS_MAX_ELEMS	(PAGINAS_CONSOLA*TAMANO_PAGINA/2 - (PAGINAS_CONSOLA*TAMANO_PAGINA/2)%CONS_ANCHO)

#define CONS_MAX_LINE 64

enum cons_flags {
	CF_SCROLLING, CF_ENTRADA, CF_DIRTY
};

typedef struct {
	short *buffer;		// Inicio de las paginas de la consola
	int cursor;		// Posicion que usaremos para escribir
	int scroll;		// Posicion (comienzo) de la pantalla que vemos
	int flags;
	int tarea;		// Tarea que la ocupe, o -1
	char input[CONS_MAX_LINE+1];
	int input_pos;
} consola_t;

void cons_inicializar();
void cons_putch(int num_cons, char ch, char modo);
int cons_putstr(int num_cons, const char *msg, char modo);
int cons_printf(int num_cons, char modo, const char *format, ...);
int cons_vprintf(int num_cons, char modo, const char *format, va_list ap);
int cons_ngets(char *buf, int size);
void cons_cambiar(int num_cons);
void cons_actualizar();
void cons_limpiar();
void cons_haytecla(unsigned ch);
int cons_obtenerNueva(int pid);
void cons_liberarUsada(int num_cons);

// Rutinas auxiliares

static inline int CURSOR_OFFSET(const consola_t* cons) {
	// XXX: el modulo no anda como se espera con los numeros negativos!
	int tmp = cons->cursor - cons->scroll;
	while (tmp < 0)
		tmp += CONS_MAX_ELEMS;
	return tmp;
}

static inline int CURSOR_X(const consola_t* cons) {
	return (CURSOR_OFFSET(cons) % CONS_ANCHO);
}

static inline int CURSOR_Y(const consola_t* cons) {
	return (CURSOR_OFFSET(cons) / CONS_ANCHO);
}

static inline void DESPLAZAR_CURSOR(consola_t *cons, int offset) {
	// XXX: el modulo no anda como se espera con los numeros negativos!
	cons->cursor += offset;
	if (offset > 0)
		while (cons->cursor >= CONS_MAX_ELEMS)
			cons->cursor -= CONS_MAX_ELEMS;
	else
		while (cons->cursor < 0)
			cons->cursor += CONS_MAX_ELEMS;
}

#endif
