// vim:ft=c
#ifndef __MMU_H__
#define __MMU_H__

#include "klib.h"

// Paginacion

#define TAMANO_PAGINA 		0x00001000
#define TAMANO_PAGINA_4MB	0x00400000
#define MAX_MEM			0x04000000

typedef struct {
	unsigned char p:1,
	rw:1,
	us:1,
	wt:1,
	cd:1,
	a:1,
	zero:1,
	size:1,
	free:4;
	unsigned int base:20;
} __attribute__((__packed__)) pdir_entry;

typedef struct {
	unsigned char p:1,
	rw:1,
	us:1,
	wt:1,
	cd:1,
	a:1,
	dirty:1,
	zero:1,
	global:1,
	free:3;
	unsigned int base:20;
} __attribute__((__packed__)) ptbl_entry;

extern void *dirPagsKernel;

// XXX: al no separar .bss y .data no podemos mapear EXEC sin WRITE, pero igual necesitariamos paginas PAE
#define PROT_USER_READ		(SET_NUM2FLAG(MMU_KernUser))
#define PROT_USER_RW		(SET_NUM2FLAG(MMU_ReadWrite) | SET_NUM2FLAG(MMU_KernUser))
#define PROT_KERNEL_READ	(0)
#define PROT_KERNEL_RW		(SET_NUM2FLAG(MMU_ReadWrite))

enum mmu_flags {
	MMU_ReadWrite, MMU_KernUser
};

void mmu_inicializar(void);
void mmu_map(void *dirPags, void *virtual, void *fisica, unsigned attrs);
void mmu_unmap(void *dirPags, void *virtual);
void mmu_mapCont(void *dirPags, void *virtual, void *fisica, unsigned attrs, int cantidad);
void mmu_unmapCont(void *dirPags, void *virtual, int cantidad);

// Administracion de memoria basica

#define INICIO_KERNEL		0x00002000
#define CANT_KERNEL		((INICIO_PAGINAS_KERNEL - INICIO_KERNEL) / TAMANO_PAGINA)
#define INICIO_PAGINAS_KERNEL 	0x00100000
#define CANT_PAGINAS_KERNEL	((INICIO_PAGINAS_USUARIO - INICIO_PAGINAS_KERNEL) / TAMANO_PAGINA)
#define INICIO_PAGINAS_USUARIO	0x00180000
#define CANT_PAGINAS_USUARIO	((MAX_MEM - INICIO_PAGINAS_KERNEL) / TAMANO_PAGINA)

typedef struct {
	unsigned posInicial, tamElem;
	int numTotal, numLibres;
	int *map;
} bitmap_t;

void *obtener_pagina_kernel(void);
void liberar_pagina_kernel(void *dir);
void *obtener_pagina_usuario(void);
void liberar_pagina_usuario(void *dir);

// Memoria virtual para procesos

void* crear_dir_usuario(void);
void destruir_dir_usuario(void *dir);

#endif
