#include <iostream>
#include <fstream>
#include <cstring>

#include "pak.h"

using namespace std;

int main(int argc, char **argv) {
	if (argc < 3) {
		cerr << "Usage: " << argv[0] << " <pak name> file1 ...\n";
		return 1;
	}

	int numfiles = argc-2;
	char **filenames = argv+2;
	ofstream outfile(argv[1], ofstream::binary);

	// Construir encabezado
	pak_header_t header;
	pak_entry_t *entries = new pak_entry_t[numfiles];
	header.magic = PAK_MAGIC;
	header.num_entries = numfiles;
	outfile.write((char*)&header, sizeof(pak_header_t));
	// Saltar hasta la parte de los archivos en crudo (porque aun no sabemos sus tamanos ni posiciones)
	outfile.seekp(sizeof(pak_entry_t)*numfiles, ofstream::cur);

	cout << "Creating PAK with " << numfiles << " files...\n";

	int abspos = sizeof(pak_header_t) + sizeof(pak_entry_t)*numfiles;
	for (int i = 0; i < numfiles; i++) {
		ifstream infile(filenames[i]);
		// Obtener tamano del archivo
		infile.seekg(0, ifstream::end);
		int size = infile.tellg();
		infile.seekg(0);

		// Concatenar al archivo de salida
		char *buffer = new char[size];
		infile.read (buffer, size);
		outfile.write (buffer, size);
		delete[] buffer;

		// Llenar encabezado del archivo dentro del PAK
		strncpy(entries[i].name, filenames[i], PAK_MAX_NAME);
		entries[i].name[PAK_MAX_NAME-1] = '\0';
		entries[i].position = abspos;
		entries[i].size = size;
		abspos += size;

		cout << "- Adding \"" << entries[i].name << "\" of " << entries[i].size << " bytes at position " << entries[i].position << endl;
	}

	outfile.seekp(sizeof(header), ofstream::beg);
	for (int i = 0; i < numfiles; i++)
		outfile.write((char*)&entries[i], sizeof(entries[0]));

	cout << "Finished PAK with " << abspos << " bytes\n";
	return 0;
}
