#define __nostart
#include "stdlib.h"

char *strcat(char *dst, const char *src) {
	while (*dst != '\0')
		dst++;
	while (*src != '\0')
		*dst++ = *src++;
	*dst = '\0';
	return dst;
}

int strcmp(const char *s1, const char *s2) {
	for (; *s1 == *s2 && *s1 != '\0'; s1++, s2++)
		;
	return *s1 - *s2;
}

int strpartcmp(const char *s1, const char *s2) {
	for (; *s1 == *s2 && *s1 != '\0'; s1++, s2++)
		;
	if (*s1 == '\0' || *s2 == '\0')
		return 0;
	else
		return *s1 - *s2;
}

int strncmp(const char *s1, const char *s2, unsigned len) {
	for (; *s1 == *s2 && *s1 != '\0' && len > 0; s1++, s2++, len--)
		;
	return *s1 - *s2;
}

int strlen(const char *s) {
	int i = 0;
	while (s[i] != '\0')
		i++;
	return i;
}

int strlcpy(char *dst, const char *src, int size) {
	int i = 0;
	for (i = 0; src[i] != '\0' && i < size; i++)
		dst[i] = src[i];
	dst[i] = '\0';
	while (src[i] != '\0')
		i++;
	return i;
}

int printf(const char *format, ...) {
	char buffer[128];
	int ret;
	va_list ap;

	va_start(ap, format);
	ret = vsnprintf(buffer, 128, format, ap);
	va_end(ap);
	sys_puts(buffer);

	return ret;
}

int snprintf(char *buffer, int size, const char *format, ...) {
	int ret;
	va_list ap;

	va_start(ap, format);
	ret = vsnprintf(buffer, size, format, ap);
	va_end(ap);

	return ret;
}

int vsnprintf(char *buffer, int size, const char *format, va_list ap) {
	const int max = size-1;
	int bufpos = 0, tmp;
	char cbuf[40];

	for (int i = 0; i < strlen(format) && bufpos < max; ) {
		if (format[i] == '%') {
			i++;
			switch (format[i]) {
			case 'b':
				itostr(va_arg(ap, int), 2, cbuf);
				tmp = strlcpy(&buffer[bufpos], cbuf, size-bufpos);
				bufpos = (tmp > size-bufpos) ? max : (bufpos+tmp);
				break;
			case 'c':
				buffer[bufpos++] = va_arg(ap, char);
				break;
			case 'd':
				itostr(va_arg(ap, int), 10, cbuf);
				tmp = strlcpy(&buffer[bufpos], cbuf, size-bufpos);
				bufpos = (tmp > size-bufpos) ? max : (bufpos+tmp);
				break;
			case 'x':
				itostr(va_arg(ap, int), 16, cbuf);
				tmp = strlcpy(&buffer[bufpos], cbuf, size-bufpos);
				bufpos = (tmp > size-bufpos) ? max : (bufpos+tmp);
				break;
			case 's':
				tmp = strlcpy(&buffer[bufpos], va_arg(ap, char*), size-bufpos);
				bufpos = (tmp > size-bufpos) ? max : (bufpos+tmp);
				break;
			case '%':
				buffer[bufpos++] = format[i];
				break;
			default:
				// Se ignora
				break;
			}
			i++;
		} else {
			buffer[bufpos++] = format[i++];
		}
	}
	buffer[bufpos] = '\0';
	
	return bufpos;
}

int itostr(unsigned x, int base, char *buf) {
	// XXX: no se pueden usar variables "static" en las tareas al parecer
	int i = 0, sign = 1;
	if (base < 2 || base > (10 + ('Z'-'A')))
		return -1;
	if (x == 0) {
		buf[0] = '0';
		buf[1] = '\0';
		return 0;
	}
	int sx = (int)x;
	if (base == 10 && sx < 0) {
		sign = -1;
		x = sx*-1;
	}

	// Conversion de base
	while (x != 0) {
		unsigned digit = x % base;
		if (digit < 10)
			buf[i++] = '0' + digit;
		else
			buf[i++] = 'A' + digit - 10;
		x /= base;
	}
	// Prefijo segun la base
	if (base == 2) {
		buf[i++] = 'b';
	} else if (base == 8) {
		buf[i++] = '0';
	} else if (base == 10 && sign < 0) {
		buf[i++] = '-';
	} else if (base == 16) {
		buf[i++] = 'x';
		buf[i++] = '0';
	}
	// Hacer el reverso del string
	for (int j = 0; j < i/2; j++) {
		char tmp = buf[j];
		buf[j] = buf[i-j-1];
		buf[i-j-1] = tmp;
	}
	buf[i] = '\0';
	return 0;
}

void memcpy(char* dst, const char* src, unsigned len) {
	while (len--) *dst++ = *src++;
}

void memcpyw(short* dst, const short* src, unsigned len) {
	while (len--) *dst++ = *src++;
}

void memset(char *dst, char c, unsigned len) {
	while (len--) *dst++ = c;
}

void memsetw(short *dst, short w, unsigned len) {
	while (len--) *dst++ = w;
}

void memsetdw(int *dst, int dw, unsigned len) {
	while (len--) *dst++ = dw;
}

int strtoi(const char *str, int base) {
	if (*str == '\0')
		return 0;
	if (base < 2 || base > (10 + ('Z'-'A')))
		return 0;
	int res = 0, i = 0, sign = 1;
	while (str[i] == ' ')
		i++;
	if (str[i] == '+')
		i++;
	else if (str[i] == '-') {
		sign = -1;
		i++;
	}
	while (str[i] != '\0') {
		char digit = str[i];
		if (digit >= '0' && digit <= '9')
			digit -= '0';
		else if (digit >= 'A' && digit < ('A' + base - 10))
			digit = digit - 'A' + 10;
		else
			return 0;
		if (digit >= base)
			return 0;
		res = res*base + digit;
		i++;
	}
	return res*sign;
}

int readInt() {
	char buffer[40];
	sys_ngets(buffer, 40);
	return strtoi(buffer, 10);
}

void srand(unsigned long long *seed) {
	*seed = rdtsc();
}

unsigned long rand(unsigned long long *seed) {
	unsigned long tmp = *seed;
	// Constantes de Numerical Recipes, segun Wikipedia
	*seed = (1664525 * *seed + 1013904223) % 0x100000000ULL;
	return tmp;
}
